@@ -13,11 +13,25 @@ NSString *CSNotSupportedException=@"CSNotSupportedException";
 
 @implementation CSHandle
 
--(id)initWithName:(NSString *)descname
+-(id)init
 {
-	if((self=[super init]))
+	if(self=[super init])
 	{
-		name=[descname retain];
+		parent=nil;
+
+		bitoffs=-1;
+
+		writebyte=0;
+		writebitsleft=8;
+	}
+	return self;
+}
+
+-(id)initWithParentHandle:(CSHandle *)parenthandle
+{
+	if(self=[super init])
+	{
+		parent=[parenthandle retain];
 
 		bitoffs=-1;
 
@@ -29,9 +43,9 @@ NSString *CSNotSupportedException=@"CSNotSupportedException";
 
 -(id)initAsCopyOf:(CSHandle *)other
 {
-	if((self=[super init]))
+	if(self=[super init])
 	{
-		name=[[[other name] stringByAppendingString:@" (copy)"] retain];
+		parent=[other->parent retain];
 
 		bitoffs=other->bitoffs;
 		readbyte=other->readbyte;
@@ -44,7 +58,7 @@ NSString *CSNotSupportedException=@"CSNotSupportedException";
 
 -(void)dealloc
 {
-	[name release];
+	[parent release];
 	[super dealloc];
 }
 
@@ -52,6 +66,7 @@ NSString *CSNotSupportedException=@"CSNotSupportedException";
 
 
 
+
 //-(off_t)fileSize { [self _raiseNotImplemented:_cmd]; return 0; }
 -(off_t)fileSize { return CSHandleMaxLength; }
 
@@ -438,34 +453,65 @@ CSWriteValueImpl(uint32_t,writeID,CSSetUInt32BE)
 -(void)_raiseMemory
 {
 	[NSException raise:CSOutOfMemoryException
-	format:@"Out of memory while attempting to read from file \"%@\" (%@).",name,[self class]];
+	format:@"Out of memory while attempting to read from file \"%@\" (%@).",
+	[self name],[self class]];
 }
 
 -(void)_raiseEOF
 {
 	[NSException raise:CSEndOfFileException
-	format:@"Attempted to read past the end of file \"%@\" (%@).",name,[self class]];
+	format:@"Attempted to read past the end of file \"%@\" (%@).",
+	[self name],[self class]];
 }
 
 -(void)_raiseNotImplemented:(SEL)selector
 {
 	[NSException raise:CSNotImplementedException
-	format:@"Attempted to use unimplemented method +[%@ %@] when reading from file \"%@\".",[self class],NSStringFromSelector(selector),name];
+	format:@"Attempted to use unimplemented method +[%@ %@] when reading from file \"%@\".",
+	[self class],NSStringFromSelector(selector),[self name]];
 }
 
 -(void)_raiseNotSupported:(SEL)selector
 {
 	[NSException raise:CSNotSupportedException
-	format:@"Attempted to use unsupported method +[%@ %@] when reading from file \"%@\".",[self class],NSStringFromSelector(selector),name];
+	format:@"Attempted to use unsupported method +[%@ %@] when reading from file \"%@\".",
+	[self class],NSStringFromSelector(selector),[self name]];
 }
 
 
--(NSString *)name { return name; }
+-(NSString *)name
+{
+	return [parent name];
+}
+
+-(CSHandle *)parentHandle
+{
+	return parent;
+}
+
+-(void)setParentHandle:(CSHandle *)newparent
+{
+	[parent autorelease];
+	parent=[newparent retain];
+}
 
 -(NSString *)description
 {
-	return [NSString stringWithFormat:@"%@ for \"%@\", position %qu",
-	[self class],name,[self offsetInFile]];
+	if(parent)
+	{
+		return [NSString stringWithFormat:@"%@ @ %qu for %@",
+		[self class],[self offsetInFile],[parent description]];
+	}
+	else if([self name])
+	{
+		return [NSString stringWithFormat:@"%@ @ %qu for \"%@\"",
+		[self class],[self offsetInFile],[self name]];
+	}
+	else
+	{
+		return [NSString stringWithFormat:@"%@ @ %qu",
+		[self class],[self offsetInFile]];
+	}
 }
 
 
