@@ -802,8 +802,21 @@
 
 	*pathptr=path;
 
-	// If we have a delegate, ask it if we should extract.
-	if(delegate) return [delegate simpleUnarchiver:self shouldExtractEntryWithDictionary:dict to:path];
+	if(delegate)
+	{
+		// If we have a delegate, ask it if we should extract.
+		if(![delegate simpleUnarchiver:self shouldExtractEntryWithDictionary:dict to:path]) return NO;
+
+		// Check if the user wants to extract the entry to his own filehandle.
+		// In such case, call into the lower-level API to run the extraction
+		// and return without doing further work.
+		CSHandle *handle=[delegate simpleUnarchiver:self outputHandleForEntryWithDictionary:dict];
+		if(handle)
+		{
+			[unarchiver runExtractorWithDictionary:dict outputHandle:handle];
+			return NO;
+		}
+	}
 
 	// Otherwise, just extract.
 	return YES;
@@ -866,6 +879,10 @@ fileFraction:(double)fileratio estimatedTotalFraction:(double)totalratio
 {
 	if(!delegate) return;
 
+	// If we receive a bogus file size ratio, give up and show estimated progress instead,
+	// as we have probably been fed a broken Zip file with 32 bit overflow.
+	if(fileratio>1) totalsize=-1;
+
 	if(totalsize>=0)
 	{
 		// If the total size is known, report exact progress.
@@ -1047,6 +1064,8 @@ fileFraction:(double)fileratio estimatedTotalFraction:(double)totalratio
 
 -(void)simpleUnarchiverNeedsPassword:(XADSimpleUnarchiver *)unarchiver {}
 
+-(CSHandle *)simpleUnarchiver:(XADSimpleUnarchiver *)unarchiver outputHandleForEntryWithDictionary:(NSDictionary *)dict { return nil; }
+
 -(NSString *)simpleUnarchiver:(XADSimpleUnarchiver *)unarchiver encodingNameForXADString:(id <XADString>)string; { return [string encodingName]; }
 
 -(BOOL)simpleUnarchiver:(XADSimpleUnarchiver *)unarchiver shouldExtractEntryWithDictionary:(NSDictionary *)dict to:(NSString *)path { return YES; }
